//# Copyright (C) 2009
//# Associated Universities, Inc. Washington DC, USA.
//#
//# This library is free software; you can redistribute it and/or modify it
//# under the terms of the GNU Library General Public License as published by
//# the Free Software Foundation; either version 2 of the License, or (at your
//# option) any later version.
//#
//# This library is distributed in the hope that it will be useful, but WITHOUT
//# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
//# License for more details.
//#
//# You should have received a copy of the GNU Library General Public License
//# along with this library; if not, write to the Free Software Foundation,
//# Inc., 675 Massachusetts Ave, Cambridge, MA 02139, USA.
//#
//# Correspondence concerning AIPS++ should be addressed as follows:
//#        Internet email: casa-feedback@nrao.edu.
//#        Postal address: AIPS++ Project Office
//#                        National Radio Astronomy Observatory
//#                        520 Edgemont Road
//#                        Charlottesville, VA 22903-2475 USA
//#

#ifndef BACKGROUNDTHREAD_H_
#define BACKGROUNDTHREAD_H_

#include <stdcasa/thread/AsynchronousTools.h>

namespace casa {

class ThreadCommunication;
class PlotMSPlot;

/**
 * Manages a thread to do a background operation and communicates
 * back to its controller with regard to work status.
 */
class BackgroundThread : public async::Thread {
public:
	BackgroundThread();
	void setClientCommunication( ThreadCommunication* threadController );
	bool getResult() const;
	virtual PlotMSPlot* getPlot();
	void* run();
	void cancel();
	virtual ~BackgroundThread();
protected:
	virtual void cancelWork();
	virtual bool doWork() = 0;
	ThreadCommunication* threadController;
	bool result;
private:
	BackgroundThread( const BackgroundThread& other );

};

} /* namespace casa */
#endif /* BACKGROUNDTHREAD_H_ */
