#!/usr/bin/env python3
import os
import sys
import setuptools
from setuptools import setup
from wheel.bdist_wheel import bdist_wheel
from datetime import datetime
from subprocess import Popen, PIPE
from distutils.util import get_platform
from distutils.core import Extension, Distribution
import platform

pyversion = float(sys.version_info[0]) + float(sys.version_info[1]) / 10.0

if pyversion < 3:
    str_encode = str
    str_decode = str
    def pipe_decode(output):
        return output
else:
    def str_encode(s):
        return bytes(s,sys.getdefaultencoding())
    def str_decode(bs):
        return bs.decode(sys.getdefaultencoding(),"strict")
    def pipe_decode(output):
        if isinstance(output,bytes) or isinstance(output,bytearray):
            return str_decode(output)
        elif isinstance(output,tuple):
            return (str_decode(output[0]),str_decode(output[1]))
        else:
            return ("","")

def compute_version( ):
    print(os.getcwd())
    proc = Popen( [ "../../scripts/version" ], stdout=PIPE, stderr=PIPE )
    out,err = pipe_decode(proc.communicate( ))
    print(out)
    devbranchtag = out.split(" ")[0].strip()
    print(devbranchtag)
    releasetag = out.split(" ")[1].strip()
    dirty=""
    if (len(out.split(" ")) == 3):
        print("Latest commit doesn't have a tag. Adding -dirty flag to version string.")
        dirty="+" + out.split(" ")[2].strip() # "+" denotes local version identifier as described in PEP440
    print(releasetag)
    devbranchversion = ""
    devbranchrevision = ""
    if (devbranchtag != releasetag):
        devbranchrevision = devbranchtag.split("-")[-1]
        if (devbranchtag.startswith("CAS-")):
            devbranchversion=devbranchtag.split("-")[1]
        else:
            devbranchversion=100
        devbranchrevision = devbranchtag.split("-")[-1]
    else:
        isDevBranch = False
    (major, minor, patch) = releasetag.split(".")
    #print(major, minor, patch, feature, devbranchversion, devbranchrevision, dirty)
    return(int(major), int(minor), int(patch), devbranchversion, devbranchrevision, dirty)

(major, minor, patch, devbranchversion, devbranchrevision, dirty) = compute_version( )
print(major, minor, patch, devbranchversion, devbranchrevision, dirty)
casaplotms_version = '%d.%d.%d%s' % (major,minor,patch,dirty)
if devbranchversion !="":
    casaplotms_version = '%d.%d.%da%s.dev%s%s' % (major,minor,patch,devbranchversion,devbranchrevision,dirty)
print(casaplotms_version)

def all_files( dir ):
    acc = [ ]
    for root, directories, filenames in os.walk(dir):
        r = os.path.join(*root.split(os.sep)[1:])
        for filename in filenames:
            acc.append(os.path.join(r,filename))
    return acc

class BinaryDistribution(Distribution):
    user_options = bdist_wheel.user_options + [
        ('build=', None, 'specify build number') ]

    def initialize_options(self):
        Distribution.initialize_options(self)
        self.build=casaplotms_version

    """Distribution which always forces a binary package with platform name"""
    def has_ext_modules(foo):
        return True

class BdistWheel(bdist_wheel):

    user_options = bdist_wheel.user_options + [
        ('build=', None, 'specify build number') ]

    def initialize_options(self):
        bdist_wheel.initialize_options(self)
        self.build=casaplotms_version

    def finalize_options(self):
        bdist_wheel.finalize_options(self)
        # Mark us as not a pure python package
        self.root_is_pure = False

    def get_tag(self):
        python, abi, plat = bdist_wheel.get_tag(self)
        # Avoid marking the package universal on arm64 installations
        if platform.machine() == 'arm64':
            plat = plat.replace("universal2","arm64")
        # We don't contain any python source
        python, abi = 'py3', 'none'
        return python, abi, plat

from setuptools.command.install import install
class InstallPlatlib(install):
    def finalize_options(self):
        install.finalize_options(self)
        if self.distribution.has_ext_modules():
            self.install_lib = self.install_platlib

setup( name="casaplotms", version=casaplotms_version,
       maintainer="Darrell Schiebel",
       maintainer_email="drs@nrao.edu",
       author="CASA development team",
       author_email="casa-feedback@nrao.edu",
       distclass=BinaryDistribution,
       url="https://open-bitbucket.nrao.edu/projects/CASA/repos/casaplotms/browse",
       download_url="https://casa.nrao.edu/download/",
       license="GNU Library or Lesser General Public License (LGPL)",
#       packages=setuptools.find_packages(),
       packages=['casaplotms','casaplotms.private', 'casaplotms.gotasks'],
       cmdclass={ 'bdist_wheel': BdistWheel,'install': InstallPlatlib },
       package_data={'casaplotms': all_files('casaplotms/__bin__') + ['LICENSE.txt']},
       install_requires=[ 'grpcio', 'protobuf', 'casatasks' ] )
